"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.ClassTransformerForArray = void 0;
// eslint-disable-next-line @typescript-eslint/ban-ts-comment
// @ts-nocheck
const class_transformer_1 = require("class-transformer");
const class_transformer_validator_1 = require("class-transformer-validator");
const storage_1 = require("class-transformer/storage");
const exceptions_1 = require("../exceptions");
const storages_1 = require("../storages");
class ClassTransformerForArray {
    plainMapValue(classType, array, options) {
        var _a, _b;
        if (!Array.isArray(array)) {
            throw new TypeError(`input is not an array: ${array}`);
        }
        const storage = (options === null || options === void 0 ? void 0 : options.arrayMemberStorage) || storages_1.defaultArrayMemberStorage;
        const map = storage.getPropertyIndex(classType);
        if (!map) {
            throw new exceptions_1.UnknownClassError(`Cannot found class of ${classType.name}`);
        }
        const obj = {};
        for (const [i, property] of map.entries()) {
            if (i in array) {
                const o = array[i];
                const typeMeta = storage_1.defaultMetadataStorage.findTypeMetadata(classType, property.key);
                const subClassType = (_a = typeMeta === null || typeMeta === void 0 ? void 0 : typeMeta.typeFunction()) !== null && _a !== void 0 ? _a : typeMeta === null || typeMeta === void 0 ? void 0 : typeMeta.reflectedType;
                if (storage.has(subClassType) && Array.isArray(o)) {
                    if ((typeMeta === null || typeMeta === void 0 ? void 0 : typeMeta.reflectedType) === Array || ((_b = property.options) === null || _b === void 0 ? void 0 : _b.isArray)) {
                        obj[property.key] = o.map((p) => this.plainMapValue(subClassType, p, options));
                    }
                    else {
                        obj[property.key] = this.plainMapValue(subClassType, o, options);
                    }
                }
                else {
                    obj[property.key] = o;
                }
            }
        }
        return obj;
    }
    classMapValue(classType, object, options) {
        var _a, _b;
        const storage = (options === null || options === void 0 ? void 0 : options.arrayMemberStorage) || storages_1.defaultArrayMemberStorage;
        const map = storage.getPropertyIndex(classType);
        if (!map) {
            throw new exceptions_1.UnknownClassError(`Cannot found class of ${classType.name}`);
        }
        const arr = [];
        for (const [i, property] of map.entries()) {
            if (property.key in object) {
                const typeMeta = storage_1.defaultMetadataStorage.findTypeMetadata(classType, property.key);
                const subClassType = (_a = typeMeta === null || typeMeta === void 0 ? void 0 : typeMeta.typeFunction()) !== null && _a !== void 0 ? _a : typeMeta === null || typeMeta === void 0 ? void 0 : typeMeta.reflectedType;
                if (storage.has(subClassType)) {
                    if ((typeMeta === null || typeMeta === void 0 ? void 0 : typeMeta.reflectedType) === Array || ((_b = property.options) === null || _b === void 0 ? void 0 : _b.isArray)) {
                        if (object[property.key] == null) {
                            arr[i] = object[property.key];
                        }
                        else if (!Array.isArray(object[property.key])) {
                            throw new TypeError(`property ${classType.name}.${property.key.toString()} is not an array: ${object[property.key]}`);
                        }
                        else {
                            arr[i] = object[property.key].map((p) => this.classMapValue(subClassType, p, options));
                        }
                    }
                    else {
                        arr[i] = this.classMapValue(subClassType, object[property.key], options);
                    }
                }
                else {
                    arr[i] = object[property.key];
                }
            }
            else {
                arr[i] = undefined;
            }
        }
        return arr;
    }
    plainArrayToClass(classType, array, options) {
        return class_transformer_1.plainToClass(classType, (options === null || options === void 0 ? void 0 : options.isArray) ? array.map((arr) => this.plainMapValue(classType, arr, options))
            : this.plainMapValue(classType, array, options), options);
    }
    classToPlainArray(object, options) {
        const record = class_transformer_1.classToPlain(object, options);
        return Array.isArray(object)
            ? record.map((r, i) => this.classMapValue(object[i].constructor, r, options))
            : this.classMapValue(object.constructor, record, options);
    }
    async arrayTransformAndValidate(classType, array, options) {
        return class_transformer_validator_1.transformAndValidate(classType, (options === null || options === void 0 ? void 0 : options.isArray) ? array.map((arr) => this.plainMapValue(classType, arr, options))
            : this.plainMapValue(classType, array, options), options);
    }
    arrayTransformAndValidateSync(classType, array, options) {
        return class_transformer_validator_1.transformAndValidateSync(classType, (options === null || options === void 0 ? void 0 : options.isArray) ? array.map((arr) => this.plainMapValue(classType, arr, options))
            : this.plainMapValue(classType, array, options), options);
    }
}
exports.ClassTransformerForArray = ClassTransformerForArray;
ClassTransformerForArray.instance = new ClassTransformerForArray();
//# sourceMappingURL=ClassTransformerForArray.js.map