"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.defaultArrayMemberClassStorage = exports.ArrayMemberClassStorage = void 0;
const PropertyIndex_1 = require("./PropertyIndex");
const ArrayMemberStorage_1 = require("./ArrayMemberStorage");
/* eslint-disable @typescript-eslint/ban-types */
class ArrayMemberClassStorage {
    constructor(defaultStorage) {
        this.map = new Map();
        this.defaultStorage = ArrayMemberStorage_1.defaultArrayMemberStorage;
        this.defaultStorage = defaultStorage || ArrayMemberStorage_1.defaultArrayMemberStorage;
    }
    add(constructor, storage) {
        storage || (storage = this.defaultStorage);
        const storages = this.initializeStorage(constructor, storage);
        return storage === this.defaultStorage
            ? this.addDefaultArrayMemberStorage(constructor)
            : this.addCustomArrayMemberStorage(constructor, storage, storages);
    }
    get(constructor) {
        return this.map.get(constructor);
    }
    has(constructor) {
        return this.map.has(constructor);
    }
    //
    initializeStorage(constructor, storage) {
        const storages = this.map.get(constructor);
        if (storages) {
            if (!storages.includes(storage)) {
                storages.push(storage);
            }
            return storages;
        }
        else {
            const storages = [storage];
            this.map.set(constructor, storages);
            return storages;
        }
    }
    addDefaultArrayMemberStorage(constructor) {
        const result = this.defaultStorage.mapMoved.get(constructor);
        if (result) {
            this.defaultStorage.map.set(constructor, result);
            this.defaultStorage.mapMoved.delete(constructor);
        }
        return this;
    }
    addCustomArrayMemberStorage(constructor, storage, storages) {
        const defaultPropertyIndex = new PropertyIndex_1.PropertyIndex();
        // existing properties
        const oldPropertyIndex = this.defaultStorage.mapMoved.get(constructor);
        if (oldPropertyIndex) {
            defaultPropertyIndex.map = new Map(oldPropertyIndex.map);
        }
        // new added
        const newPropertyIndex = this.defaultStorage.map.get(constructor);
        if (newPropertyIndex) {
            defaultPropertyIndex.map = new Map([...defaultPropertyIndex.map.entries(), ...newPropertyIndex.map.entries()]);
            // cleanup
            if (!storages.includes(this.defaultStorage)) {
                if (oldPropertyIndex) {
                    // merge new item to old
                    for (const [i, k] of newPropertyIndex.map.entries()) {
                        oldPropertyIndex.set(i, k);
                    }
                }
                else {
                    this.defaultStorage.mapMoved.set(constructor, defaultPropertyIndex);
                }
                this.defaultStorage.map.delete(constructor);
            }
        }
        // merge
        const storagePropertyIndexMap = storage.map.get(constructor);
        if (storagePropertyIndexMap) {
            for (const [i, k] of storagePropertyIndexMap.entries()) {
                defaultPropertyIndex.set(i, k);
            }
        }
        storage.map.set(constructor, defaultPropertyIndex);
        return this;
    }
}
exports.ArrayMemberClassStorage = ArrayMemberClassStorage;
ArrayMemberClassStorage.defaultArrayMemberClassStorage = new ArrayMemberClassStorage();
exports.defaultArrayMemberClassStorage = ArrayMemberClassStorage.defaultArrayMemberClassStorage;
//# sourceMappingURL=ArrayMemberStorageClass.js.map